#
# "@(#) $Id: CDROM.py,v 1.5 2004/08/18 20:19:51 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
import os, fcntl, array
from CDLibrary import *
from LMusicCDMounter import *
from LMusicBurner import *

#
# the singleton instance of the CDROM drive
#

class CDROM(QObject):
	def __init__(self):
		QObject.__init__(self)
		self._cd = True # check for availability of CD
		self._hasCD = False
		self._numTracks = 0
		#self.startTimer()

	#
	# set up timer to check whether or not a CD has been inserted or ejected
	#
	def startTimer(self):
		self.timer = QTimer()
		self.timer.start(4000)
		self.connect(self.timer,SIGNAL("timeout()"),self.checkStatus)

	#
	# check the status of the drive - if the state changes, generate a signal
	#
	def checkStatus(self):
		if self._cd:
			hasCD = self.checkForMedia()
			if hasCD!=self._hasCD:
				if hasCD:
					self.timer.stop()
					CDLibrary.clobber()
					self.emit(PYSIGNAL("cdInserted"),(None,None))
					LMusicCDMounter.mount(CDLibrary.singleton())
					self.timer.start(4000)
				else:
					self.emit(PYSIGNAL("cdEjected"),(None,None))
				self._hasCD = hasCD

	def checkForMedia(self):
		#print "checking CD"
		if LMusicBurner.singleton().isBurning():
			return False
		try:
			isValidCD = False
			fd = os.open('/dev/cdrom',os.O_RDONLY | os.O_NONBLOCK)
			s = fcntl.ioctl(fd,0x5326) # check if tray closed, media present
			if s==4:
				s = fcntl.ioctl(fd,0x5327) # check media type
				isValidCD = (s==100 or s==105) # valid if audio or mixed
				if isValidCD:
					self._numTracks = self.getNumTracks(fd)
			os.close(fd)
			return isValidCD
		except:
			try: os.close(fd)
			except: pass
			return False
	
	def getNumTracks(self,fd):
		if LsongsPlatform=='Lindows':
			# the old way of doing ioctl with results
			s = fcntl.ioctl(fd,0x5305,'  ')
			return ord(s[1])
		else:
			# the new way of doing ioctl with results
			a = array.array('b',[0,0])
			s = fcntl.ioctl(fd,0x5305,a,1)
			if s==0:
				return a[1]

	def getTocEntry(self,fd,index):
		if LsongsPlatform=='Lindows':
			s = fcntl.ioctl(fd,0x5306,'        ')
		else:
			a = array.array('b',[0,0,0,0,0,0,0,0])
			s = fcntl.ioctl(fd,0x5306,a,1)
			if s==0:
				pass

	def numTracks(self):
		return self._numTracks

	def hasCD(self):
		return self._hasCD

	def cd(self):
		return self._cd

	#
	# attempt to eject the currently mounted CD
	#
	def eject(self):
		if self._cd:
			try:
				self.ejectMedia()
			except:
				KMessageBox.error(None,i18n("Cannot eject CD from CDROM drive"),i18n("Cannot Eject CD"))
				return False
		return True


	def ejectMedia(self):
		try:
			fd = os.open('/dev/cdrom',os.O_RDONLY | os.O_NONBLOCK)
			fcntl.ioctl(fd,0x5309)
			os.close(fd)
		except: pass

	#
	# return the singleton instance of the CDROM drive
	#
	def static_singleton():
		global _CDROMSingleton
		return _CDROMSingleton
	singleton = staticmethod(static_singleton)

_CDROMSingleton = CDROM()
