#
# "@(#) $Id: DellLibrary.py,v 1.5 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from Library import *
from CDTrack import *
from LMusicPlayer import *
from LsongsDevices import *
from DellTrack import *
from PListParser import *
#from LMusicDellMounter import *
import os
import warnings
import backtick

#
# the singleton instance of the Dell DJ library
#
_DellLibrary = None

class DellLibrary(Library):
	def __init__(self):
		Library.__init__(self)
		self.name = "Dell Digital Jukebox"
		self.dirty = True
		self.addNewPlaylist("NJB")
		self.info = None

	def readInfo(self):
		info = backtick.backtick('njbgetinfo')
		reader = PListReader()
		info = reader.parseString(info)
		self.info = info[0]
	
	def readPlaylists(self):
		playlists = backtick.backtick('njbplaylists')
		reader = PListReader()
		playlists = reader.parseString(playlists)
		self.playlists = playlists[0]
	
	def readTracks(self,thread = None):
		while True:
			trackDescs = backtick.backtick('njbtracks')
			cmd = thread.getCommand()
			if cmd=='stop':
				return False
			if not re.match(r'\s*?Operation not permitted',trackDescs):
				self.fromText(trackDescs)
				break
			else:
				if thread!=None:
					thread.msleep(3000)
		return True

	def fromText(self,trackDescs):
		trackDescs = unicode(trackDescs,'latin-1').encode('utf-8')
		reader = PListReader()
		trackDescs = reader.parseString(trackDescs)
		try:
			trackDescs = trackDescs[0]
			#print "descs",trackDescs
			index = 1
			for trackKey in trackDescs.keys():
				trackDesc = trackDescs[trackKey]
				#print "loading",trackDesc
				track = DellTrack(self)
				trackDesc['ID'] = int(trackKey)
				track.fromDellStruct(trackDesc)
				track.trackNum = index
				self.addTrack(track)
				index = index + 1				
			self.emit(PYSIGNAL("readTracks"),(self,None))
		except: pass
	
	#
	# creates a dictionary of tracks keyed with title/artist/album string
	#
	def hashList(self):
		result = {}
		for trackID in self._tracks.keys():
			track = self._tracks[trackID]
			result[track.hashKey()] = track
		return result
	
	#
	# prepare for a synchronization between the DDJ and
	# the Lsongs database:
	#
	# 1) Create a "remove" hash based on the title/artist/album of each track in the Dell
	# 2) for each track in the Lsongs library
	#    a) remove the track from the remove hash if it's there
	#    b) add the track to an "add" hash if it isn't
	#	 c) handle duplicates!
	#
	def getSyncInfoWith(self,library):
		removes = self.hashList()
		adds = {}
		removed = {}
		
		for trackID in library._tracks.keys():
			track = library._tracks[trackID]
			if track.kind in ['MPEG audio file']: #,'WAV audio file']:
				key = track.hashKey()
				if removes.has_key(key):
					#print "removing",key
					removed[key] = track
					del removes[key]
				else:
					#print "adding",key
					if not adds.has_key(key) and not removed.has_key(key): # handle dups
						adds[key] = track
					#else: print "duplicate!"
		return (adds,removes)
	#
	# return the singleton instance of the library corresponding to the CD
	#
	def static_singleton():
		global _DellLibrary
		if _DellLibrary==None:
			_DellLibrary = DellLibrary()
		return _DellLibrary

	singleton = staticmethod(static_singleton)
	
	def static_clobber():
		DellLibrary.singleton().clearTracks()
	
	clobber = staticmethod(static_clobber)

	def static_mount():
		from LMusicDellMounter import LMusicDellMounter
		DellLibrary.clobber()
		LMusicDellMounter.mount(DellLibrary.singleton())
	
	mount = staticmethod(static_mount)
