import os,re,time
import backtick
from qt import *

class FileBasedPlayerScanner(QObject):
	def __init__(self):
		QObject.__init__(self)
		self.lastScan = {}
		self.timer = QTimer()
		QObject.connect(self.timer,SIGNAL("timeout()"),self.doScan)
		self.timer.start(10000)
	
	def doScan(self):
		players = self.scanForPlayers()
		for player in players:
			if not self.lastScan.has_key(player):
				player.mount()
		for player in self.lastScan:
			if not players.has_key(player):
				player.unmount()
		self.lastScan = players

	def scanForPlayers(self):
		players = {}
		mountRoots = [os.path.expanduser('~'),'/mnt']
		mountDevices = backtick.backtick('mount')
		for mountRoot in mountRoots:
			if os.access(mountRoot,os.R_OK) and os.path.isdir(mountRoot):
				try:
					mountPoints = os.listdir(mountRoot)
					for mountPoint in mountPoints:
						if not mountPoint in ['floppy','cdrom']:
							mountPath = os.path.join(mountRoot,mountPoint)
							if os.access(mountPath,os.R_OK) and os.path.isdir(mountPath):
								mountPattern = re.compile(r"%s.*,dev=(.*?),"% mountPath, re.MULTILINE)
								mo = mountPattern.search(mountDevices)
								if mo:
									devicePath = mo.group(1)
									scsiPath = os.readlink(str(devicePath))
									mountPattern = re.compile(r"scsi/host(\d*)/bus(\d*)/target(\d*)/lun(\d*)/part(\d*)")
									mo = mountPattern.match(scsiPath)
								else:
									mountPattern = re.compile(r"/dev/scsi/host(\d*)/bus(\d*)/target(\d*)/lun(\d*)/part(\d*) on %s" % mountPath, re.MULTILINE)
									mo = mountPattern.match(mountDevices)
								if mo:
									scsiPattern = re.compile(r"scsi%d Channel:\s*%02d Id:\s*%02d.*\s*Vendor:\s*(.*?)\s*Model:\s*(.*?)\s*Rev:\s*" % (int(mo.group(1)), int(mo.group(2)), int(mo.group(3))), re.MULTILINE)
									# Vendor: Casio    Model: QV DigitalCamera Rev: 1000
									scsiLines = open("/proc/scsi/scsi").read()
									mo = scsiPattern.search(scsiLines, re.MULTILINE)
									if mo:
										vendor = mo.group(1)
										model = mo.group(2)
										className = '%s%sPlayer' % (vendor,model)
										#print className
										try:
											klass = eval(className)
											if klass.recognize(mountPath):
												player = klass(mountPath,vendor,model)
												player = {'MountPath':mountPath,'Vendor':vendor,'Model':model}
												players[klass] = player
										except:
											pass
				except:
					pass
		return players
		
	def static_singleton():
		global _fileBasedPlayerScannerSingleton
		if _fileBasedPlayerScannerSingleton==None:
			_fileBasedPlayerScannerSingleton = FileBasedPlayerScanner()
		return _fileBasedPlayerScannerSingleton
		
	singleton = staticmethod(static_singleton)

_fileBasedPlayerScannerSingleton = FileBasedPlayerScanner()

class FileBasedPlayer:
	def __init__(self,mountPath,vendor,model):
		self.mountPath = mountPath
		self.vendor = vendor
		self.model = model

class RioCaliPlayer(FileBasedPlayer):
	def __init__(self,mountPath,vendor,model):
		FileBasedPlayer.__init__(self,mountPath,vendor,model)

	def static_recognize(mountPath):
		return True	
	recognize = staticmethod(static_recognize)

	def static_mount():
		print "mounting Rio Cali"
	mount = staticmethod(static_mount)
	
	def static_unmount():
		print "unmounting Rio Cali"
	unmount = staticmethod(static_unmount)

if __name__=='__main__':
	players = FileBasedPlayerScanner.singleton().scanForPlayers()
	for player in players.keys():
		print player,players[player]
