#
# "@(#) $Id: LArtworkView.py,v 1.3 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from qt import *
from LArtworkWindow import *

class LArtworkView(QWidget):
	def __init__(self,container,name,parent):
		QWidget.__init__(self,container,name)
		self.parent = parent
		self.fullPixmap = None
		self.smallPixmap = None
		self.hasImage = False
		self.currentScale = 0
		self.setPaletteBackgroundColor(QColor(255,255,255))
		self.setBackgroundMode(Qt.NoBackground)
		self.setSizePolicy(QSizePolicy(QSizePolicy.Preferred,QSizePolicy.Preferred,True))

	#
	# force us to be square up to half the height of our parent
	#
	def heightForWidth(self,w):
		parentMiddle = self.parent.height()/2
		if w>parentMiddle:
			return parentMiddle
		return w

	#
	# recompute the small pixmap as we resize
	#
	def resizeEvent(self,e):
		QWidget.resizeEvent(self,e)
		self.buildSmallPixmap()

	#
	# get the song's artwork and build a scaled version for quicker drawing
	#
	def fetchItemArtwork(self,item):
		if item == None:
			return
		self.track = item.track()
		data = self.track.image
		if data != None:
			self.fullPixmap = QPixmap()
			self.fullPixmap.loadFromData(data)
			self.currentScale = 0
			self.buildSmallPixmap()
			self.hasImage = True
			self.setEnabled(True)
		else:
			self.fullPixmap = None
			self.buildSmallPixmap()
			self.hasImage = False
			self.setEnabled(False)

	#
	# build a scaled version of the full sized image and keep it
	# maintain the aspect ratio, and don't increase the size
	#
	def buildSmallPixmap(self):
		if self.fullPixmap:
			pixSize = self.fullPixmap.size()
			size = self.size()
			width = pixSize.width()
			height = pixSize.height()
			if width==0: width = 1
			if height==0: height = 1
			scaleX = 1.0*size.width()/width
			scaleY = 1.0*size.height()/height
			scale = scaleX
			if scaleX>scaleY:
				scale = scaleY
			if scale>1.0:
				scale = 1.0
			if scale != self.currentScale:
				mat = QWMatrix()
				mat.scale(scale,scale)
				self.smallPixmap = self.fullPixmap.xForm(mat)
				self.currentScale = scale
		else:
			self.smallPixmap = QPixmap()
		self.repaint()

	#
	# paint the cached scaled image of the cover in the artwork area
	#
	# to avoid flicker, we've turned off erasing, so we're responsible for
	# clearing out the emptry areas around the picture
	#
	def paintEvent(self,e):
		p = QPainter(self)
		p.setBrush(QColor(255,255,255))
		p.setPen(QColor(255,255,255))
		srcSize = self.smallPixmap.size()
		dstSize = self.size()
		posX = (dstSize.width()-srcSize.width())/2
		posY = (dstSize.height()-srcSize.height())/2
		bitBlt(self,QPoint(posX,posY),self.smallPixmap,self.smallPixmap.rect(),Qt.CopyROP)
		if posX:
			r = QRect(0,0,posX,dstSize.height())
			p.drawRect(r)
			r = QRect(dstSize.width()-posX,0,posX,dstSize.height())
			p.drawRect(r)
		if posY:
			r = QRect(0,0,dstSize.width(),posY)
			p.drawRect(r)
			r = QRect(0,dstSize.height()-posY,dstSize.width(),posY)
			p.drawRect(r)

	def mouseDoubleClickEvent(self,ev):
		self.openInWindow()
	
	def openInWindow(self):
		self.art = LArtworkWindow(self.fullPixmap,self.track.title)
