#
# "@(#) $Id: LGetInfoDialog.py,v 1.8 2004/08/18 20:19:51 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from musicgenres import *
from human import *
from presets import *
from utils import *
from LRatingWidget import *

def fittedPixmap(pixmap,width,height):
	pixSize = pixmap.size()
	pw = pixSize.width()
	ph = pixSize.height()
	if pw==0: pw = 1
	if ph==0: ph = 1
	scaleX = 1.0*width/pw
	scaleY = 1.0*height/ph
	scale = scaleX
	if scaleX>scaleY:
		scale = scaleY
	if scale>1.0:
		scale = 1.0
	mat = QWMatrix()
	mat.scale(scale,scale)
	newpix = pixmap.xForm(mat)
	return newpix

def _get(a,b):
	if a==None:
		return b
	return a

class LArtIconView(KIconView):
	_validExtensions = ['.jpg','.png']

	def __init__(self,container,name=''):
		KIconView.__init__(self,container,name)
		self.setAcceptDrops(True)
	
	def printFormats(self,event):
		print("formats:")
		n = 0
		while True:
			format = event.format(n)
			if (format==None):
				break
			print("  "+str(format))
			n = n+1

	def canImportFile(self,filePath):
		if os.path.exists(str(filePath)):
			if os.path.isdir(str(filePath)):
				return False
			else:
				ext = os.path.splitext(str(filePath))[1]
				if ext in self._validExtensions:
					return True
		return False

	def canImportFiles(self,uris):
		for uri in uris:
			if self.canImportFile(uri):
				return True
		return False

	def dragEnterEvent(self,event):
		print "drag enter"
		if QUriDrag.canDecode(event):
			uris = QStringList()
			if QUriDrag.decodeLocalFiles(event,uris):
				if self.canImportFiles(uris):
					event.accept(True)
		#KIconView.dragEnterEvent(self,event)

	def dropEvent(self,e):
		print "drag drop"
		#KIconView.dropEvent(self,e)

class LGetInfoDialog(KDialogBase):
	_numbers = "0123456789"

	def __init__(self,track):
		self.track = track
		trackName = _get(self.track.title,'<untitled>')
		KDialogBase.__init__(self,KDialogBase.Tabbed,trackName,KDialogBase.Ok+KDialogBase.Cancel,\
			KDialogBase.Ok,None,"getinfo",True)
		self.add()
		self.load()
		self.adjustSize()
	
	def add(self):
		self.addInfo()
		self.addSummary()
		self.addOptions()
		self.addArtwork()
	
	def load(self):
		self.loadSummary()
		self.loadInfo()
		self.loadOptions()
		self.loadArtwork()

	def unload(self):
		tags = {}
		self.unloadSummary(tags)
		self.unloadInfo(tags)
		self.unloadOptions(tags)
		self.unloadArtwork(tags)
		return tags

	def addSummary(self):
		page = self.addPage(i18n("Summary"))
		layout = QVBoxLayout(page)
		self.addSummaryArtworkAndTitle(page,layout)
		self.addHDivider(page,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
		self.addSummaryInfo(page,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
		self.addHDivider(page,layout)
		self.addSummaryLocation(page,layout)

	def addSummaryArtworkAndTitle(self,container,containerLayout):
		layout = QHBoxLayout()
		containerLayout.addLayout(layout)
		self.addSummaryArtwork(container,layout)
		layout.addItem(QSpacerItem(5,1,QSizePolicy.Fixed,QSizePolicy.Fixed))
		self.addSummaryTitle(container,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))

	def getAlbumCover(self):
		data = self.track.image
		if data==None:
			return None
		pixmap = QPixmap()
		pixmap.loadFromData(data)
		return pixmap
		
	def addSummaryArtwork(self,container,containerLayout):
		self.artwork = QLabel("",container)
		self.artwork.setFrameStyle(QFrame.Box+QFrame.Plain)
		self.artwork.setLineWidth(1)
		self.artwork.setFixedSize(QSize(100,100))
		self.artwork.setPaletteBackgroundColor(QColor(255,255,255))
		self.artwork.setSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed)
		self.artwork.setAlignment(Qt.AlignCenter)
		containerLayout.addWidget(self.artwork)
	
	def addSummaryTitle(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		self.nameDuration = QLabel("",container)
		self.artist = QLabel("",container)
		self.album = QLabel("",container)
		layout.addWidget(self.nameDuration)
		layout.addWidget(self.artist)
		layout.addWidget(self.album)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))

	def addSummaryInfo(self,container,containerLayout):
		layout = QHBoxLayout()
		containerLayout.addLayout(layout)
		grid = QGridLayout(7,2)
		layout.addLayout(grid)

		label = QLabel(i18n("<b>Kind:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,0,0)
		self.kind = QLabel("",container)
		grid.addWidget(self.kind,0,1)

		label = QLabel(i18n("<b>Size:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,1,0)
		self.fileSize = QLabel("",container)
		grid.addWidget(self.fileSize,1,1)

		label = QLabel(i18n("<b>Bit Rate:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,2,0)
		self.bitRate = QLabel("",container)
		grid.addWidget(self.bitRate,2,1)

		label = QLabel(i18n("<b>Sample Rate:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,3,0)
		self.sampleRate = QLabel("",container)
		grid.addWidget(self.sampleRate,3,1)

		label = QLabel(i18n("<b>Date Modified:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,4,0)
		self.modDate = QLabel("",container)
		grid.addWidget(self.modDate,4,1)

		label = QLabel(i18n("<b>Play Count:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,5,0)
		self.playCount = QLabel("",container)
		grid.addWidget(self.playCount,5,1)

		label = QLabel(i18n("<b>Last Played:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,6,0)
		self.lastPlayed = QLabel("",container)
		grid.addWidget(self.lastPlayed,6,1)

		layout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))

		grid = QGridLayout(7,2)
		layout.addLayout(grid)

		label = QLabel(i18n("<b>Format:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,0,0)
		self.format = QLabel("",container)
		grid.addWidget(self.format,0,1)

		label = QLabel(i18n("<b>Channels:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,1,0)
		self.channels = QLabel("",container)
		grid.addWidget(self.channels,1,1)

		label = QLabel(i18n("<b>ID3 Tag:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,2,0)
		self.id3tag = QLabel("",container)
		grid.addWidget(self.id3tag,2,1)

		label = QLabel(i18n("<b>Encoded with:</b>"),container)
		label.setAlignment(Qt.AlignRight)
		grid.addWidget(label,3,0)
		self.encoder = QLabel("",container)
		grid.addWidget(self.encoder,3,1)
		
	def addSummaryLocation(self,container,containerLayout):
		layout = QHBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(i18n("<b>Where:</b>"),container)
		self.location = KSqueezedTextLabel(container)
		#self.location.setText(self.track.get("Location","<file not found>"))
		layout.addWidget(label)
		layout.addWidget(self.location)

	def loadSummary(self):
		pixmap = self.getAlbumCover()
		if pixmap:
			self.artwork.setText("")
			self.artwork.setPixmap(fittedPixmap(pixmap,100,100))
		else:
			self.artwork.setPixmap(QPixmap())
			self.artwork.setText(u"<font color='#C0C0C0'><b>%s</b></font>" % unikode(i18n("No Artwork<br>Available")))
		unknown = i18n('(unknown)')
		self.nameDuration.setText(self.track.title+"  ("+humanDuration(self.track.totalTime)+")")
		self.artist.setText(_get(self.track.artist,i18n("Unknown Artist")))
		self.album.setText(_get(self.track.album,i18n('Unknown Album')))
		self.kind.setText(_get(self.track.kind,unknown))
		self.fileSize.setText(humanFileSize(self.track.size))
		self.bitRate.setText(humanBitRate(self.track.bitRate))
		self.sampleRate.setText(humanSampleRate(self.track.sampleRate))
		self.modDate.setText(humanTimeStamp(self.track.dateModified))
		self.playCount.setText(str(self.track.playCount))
		self.lastPlayed.setText(humanTimeStamp(self.track.playDate))
		self.format.setText(_get(self.track.format,unknown))
		self.channels.setText(str(_get(self.track.mode,_get(self.track.channels,unknown))))
		self.id3tag.setText(_get(self.track.id3Version,unknown))
		self.encoder.setText(_get(self.track.encodedBy,unknown))
		self.location.setText(_get(self.track.location,i18n("(File Not Found)")))

	def unloadSummary(self,tags):
		pass

	def addInfo(self):
		page = self.addPage(i18n("Info"))
		layout = QVBoxLayout(page)
		self.addInfoItems(page,layout)

	def addInfoItems(self,container,containerLayout):
		label = QLabel(i18n("<b>Name</b>"),container)
		self.nameEdit = KLineEdit("",container)
		containerLayout.addWidget(label)
		containerLayout.addWidget(self.nameEdit)

		gLayout = QHBoxLayout()
		containerLayout.addLayout(gLayout)

		aagLayout = QVBoxLayout()
		gLayout.addLayout(aagLayout,3)

		label = QLabel(i18n("<b>Artist</b>"),container)
		self.artistEdit = KLineEdit("",container)
		aagLayout.addWidget(label)
		aagLayout.addWidget(self.artistEdit)

		label = QLabel(i18n("<b>Album</b>"),container)
		self.albumEdit = KLineEdit("",container)
		aagLayout.addWidget(label)
		aagLayout.addWidget(self.albumEdit)

		label = QLabel(i18n("<b>Grouping</b>"),container)
		self.groupingEdit = KLineEdit("",container)
		aagLayout.addWidget(label)
		aagLayout.addWidget(self.groupingEdit)

		label = QLabel(i18n("<b>Composer</b>"),container)
		self.composerEdit = KLineEdit("",container)
		containerLayout.addWidget(label)
		containerLayout.addWidget(self.composerEdit)

		label = QLabel(i18n("<b>Comment</b>"),container)
		self.commentEdit = KTextEdit(container)
		containerLayout.addWidget(label)
		containerLayout.addWidget(self.commentEdit)

		label = QLabel(i18n("<b>Genre</b>"),container)
		containerLayout.addWidget(label)

		gcLayout = QHBoxLayout()
		containerLayout.addLayout(gcLayout)

		self.genreEdit = KComboBox(False,container)
		sortedGenres = musicGenres[:]
		sortedGenres.sort()
		for genre in sortedGenres:
			self.genreEdit.insertItem(genre)
		gcLayout.addWidget(self.genreEdit,2)

		self.compilationEdit = QCheckBox(i18n("Part of a compilation"),container)
		gcLayout.addWidget(self.compilationEdit,1)
		gcLayout.insertSpacing(1,10)

		ytdLayout = QVBoxLayout()
		gLayout.addLayout(ytdLayout,1)
		gLayout.insertSpacing(1,10)

		label = QLabel(i18n("<b>Year</b>"),container)
		self.yearEdit = KRestrictedLine(container,"yearEdit",self._numbers)
		ytdLayout.addWidget(label)
		ytdLayout.addWidget(self.yearEdit)

		label = QLabel(i18n("<b>Track Number</b>"),container)
		ytdLayout.addWidget(label)
		
		layout = QHBoxLayout()
		ytdLayout.addLayout(layout)
		self.trackNumEdit = KRestrictedLine(container,"trackNumEdit",self._numbers)
		layout.addWidget(self.trackNumEdit)
		label = QLabel(i18n(" of "),container)
		layout.addWidget(label)
		self.trackCountEdit = KRestrictedLine(container,"trackCountEdit",self._numbers)
		layout.addWidget(self.trackCountEdit)

		label = QLabel(i18n("<b>Disc Number</b>"),container)
		ytdLayout.addWidget(label)

		layout = QHBoxLayout()
		ytdLayout.addLayout(layout)
		self.discNumEdit = KRestrictedLine(container,"discNumEdit",self._numbers)
		layout.addWidget(self.discNumEdit)
		label = QLabel(i18n(" of "),container)
		layout.addWidget(label)
		self.discCountEdit = KRestrictedLine(container,"discCountEdit",self._numbers)
		layout.addWidget(self.discCountEdit)

	def loadInfo(self):
		self.nameEdit.setText(self.track.title)
		self.artistEdit.setText(self.track.artist)
		self.albumEdit.setText(self.track.album)
		self.groupingEdit.setText(self.track.grouping)
		self.composerEdit.setText(self.track.composer)
		self.commentEdit.setText(self.track.comment)
		self.genreEdit.setCurrentText(self.track.genre)
		self.yearEdit.setText(str(_get(self.track.year,"")))
		self.trackNumEdit.setText(str(_get(self.track.trackNum,"")))
		self.trackCountEdit.setText(str(_get(self.track.trackCount,"")))
		self.discNumEdit.setText(str(_get(self.track.discNum,"")))
		self.discCountEdit.setText(str(_get(self.track.discCount,"")))
		self.compilationEdit.setChecked(_get(self.track.compilation,False))

	def unloadInfo(self,tags):
		if self.nameEdit.edited(): tags['title'] = str(self.nameEdit.text())
		if self.artistEdit.edited(): tags['artist'] = str(self.artistEdit.text())
		if self.albumEdit.edited(): tags['album'] = str(self.albumEdit.text())
		if self.groupingEdit.edited(): tags['grouping'] = str(self.groupingEdit.text())
		if self.composerEdit.edited(): tags['composer'] = str(self.composerEdit.text())
		if self.track.genre!= str(self.genreEdit.currentText()):
			tags['genre'] = str(self.genreEdit.currentText())
		if self.commentEdit.isModified(): tags['comment'] = str(self.commentEdit.text())
		if self.yearEdit.edited(): tags['year'] = str(self.yearEdit.text())
		if self.trackNumEdit.edited(): tags['trackNum'] = str(self.trackNumEdit.text())
		if self.trackCountEdit.edited(): tags['trackCount'] = str(self.trackCountEdit.text())
		if self.discNumEdit.edited(): tags['discNum'] = str(self.discNumEdit.text())
		if self.discCountEdit.edited(): tags['discCount'] = str(self.discCountEdit.text())
		if self.compilationEdit.isChecked() != self.track.compilation:
			tags['compilation']=self.compilationEdit.isChecked()

	def addOptions(self):
		page = self.addPage(i18n("Options"))
		layout = QVBoxLayout(page)
		self.addOptionsItems(page,layout)

	def addOptionsItems(self,container,containerLayout):
		gLayout = QGridLayout(5,2)
		containerLayout.addLayout(gLayout)
		gLayout.setSpacing(10)

		label = QLabel(i18n("Volume Adjustment:"),container)
		label.setAlignment(Qt.AlignRight)
		gLayout.addWidget(label,0,0)

		self.volumeSlider = QSlider(Qt.Horizontal,container)
		self.volumeSlider.setMinValue(-100)
		self.volumeSlider.setMaxValue(100)
		self.volumeSlider.setTickmarks(QSlider.Below)
		self.volumeSlider.setTickInterval(10)
		gLayout.addWidget(self.volumeSlider,0,1)		

		label = QLabel(i18n("Equalizer Preset:"),container)
		label.setAlignment(Qt.AlignRight)
		gLayout.addWidget(label,1,0)

		self.presetEdit = KComboBox(False,container)
		for preset in equalizerPresets:
			self.presetEdit.insertItem(preset[0])
		gLayout.addWidget(self.presetEdit,1,1)

		label = QLabel(i18n("My Rating:"),container)
		label.setAlignment(Qt.AlignRight)
		gLayout.addWidget(label,2,0)

		self.ratingEdit = LRatingWidget(container)
		gLayout.addWidget(self.ratingEdit,2,1)

		self.startTimeCheck = QCheckBox(i18n("Start Time:"),container)
		gLayout.addWidget(self.startTimeCheck,3,0)

		self.startTimeEdit = KRestrictedLine(container,"startTimeEdit",self._numbers+":.")
		gLayout.addWidget(self.startTimeEdit,3,1)

		self.endTimeCheck = QCheckBox(i18n("End Time:"),container)
		gLayout.addWidget(self.endTimeCheck,4,0)

		self.endTimeEdit = KRestrictedLine(container,"endTimeEdit",self._numbers+":.")
		gLayout.addWidget(self.endTimeEdit,4,1)

	def loadOptions(self):
		self.volumeSlider.setValue(_get(self.track.volume,0))
		self.presetEdit.setCurrentText(self.track.preset)
		self.ratingEdit.setRating(self.track.rating)
		self.startTimeEdit.setText(str(_get(self.track.startTime,"")))
		self.endTimeEdit.setText(str(_get(self.track.endTime,"")))
	
	def unloadOptions(self,tags):
		if _get(self.track.volume,0)!=self.volumeSlider.value():
			tags['volume'] = self.volumeSlider.value()
		if self.track.preset!= str(self.presetEdit.currentText()):
			tags['preset'] = str(self.presetEdit.currentText())
		if self.ratingEdit.edited(): tags['rating'] = self.ratingEdit.rating()
		if self.startTimeEdit.edited(): tags['startTime'] = str(self.startTimeEdit.text())
		if self.endTimeEdit.edited(): tags['endTime'] = str(self.endTimeEdit.text())

	def addArtwork(self):
		page = self.addPage(i18n("Artwork"))
		layout = QVBoxLayout(page)
		self.addArtworkItems(page,layout)
		layout.setSpacing(10)

	def addArtworkItems(self,container,containerLayout):
		self.artIconView = LArtIconView(container,"")
		containerLayout.addWidget(self.artIconView)
		hbox = QHBoxLayout()
		hbox.setSpacing(5)
		containerLayout.addLayout(hbox)
		self.addButton = QPushButton(i18n("Add..."),container)
		hbox.addWidget(self.addButton)
		self.deleteButton = QPushButton(i18n("Delete"),container)
		hbox.addWidget(self.deleteButton)
		hbox.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		self.sizeSlider = QSlider(Qt.Horizontal,container)
		hbox.addWidget(self.sizeSlider)
	
	def loadArtwork(self):
		pixmap = self.getAlbumCover()
		if pixmap:
			item = QIconViewItem(self.artIconView,"",pixmap)
			item.setDragEnabled(True)

	def unloadArtwork(self,tags):
		pass

	def addHDivider(self,container,containerLayout):
		containerLayout.addItem(QSpacerItem(1,2,QSizePolicy.Expanding,QSizePolicy.Fixed))
		w = QWidget(container)
		w.setPaletteBackgroundColor(QColor(0,0,0))
		w.setFixedHeight(1)
		w.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Fixed)
		containerLayout.addWidget(w)
		containerLayout.addItem(QSpacerItem(1,2,QSizePolicy.Expanding,QSizePolicy.Fixed))
