#
# "@(#) $Id: LGetMultiInfoDialog.py,v 1.5 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from musicgenres import *
from presets import *
import sys
from LRatingWidget import *
from utils import *

class LGetMultiInfoCheckBox(QCheckBox):
	def __init__(self,container):
		QCheckBox.__init__(self,container)
	
	def setLineEditItem(self,item):
		QObject.connect(item,SIGNAL("textChanged(const QString &)"),self.itemChanged)
	
	def setComboBoxItem(self,item):
		QObject.connect(item,SIGNAL("activated(const QString &)"),self.itemChanged)

	def setSliderItem(self,item):
		QObject.connect(item,SIGNAL("valueChanged(int)"),self.itemChanged)
	
	def setRatingItem(self,item):
		QObject.connect(item,PYSIGNAL("ratingChanged"),self.itemChanged)

	def itemChanged(self):
		self.setChecked(True)

class LGetMultiInfoDialog(KDialogBase):
	def __init__(self,tracks = None):
		self.tracks = tracks
		caption = i18n("Multiple Song Information")
		KDialogBase.__init__( self, None, "getmultiinfodialog", True, caption, KDialogBase.Ok+KDialogBase.Cancel, KDialogBase.Ok, True )
		self.build()
		self.load()
		self.adjustSize()
		self.setMinimumSize(self.size())
	
	def build(self):
		page = QWidget(self)
		self.setMainWidget(page)
		layout = QVBoxLayout(page)
		#self.layout().addLayout(layout)
		self.buildUpperItems(page,layout)
		self.buildLowerItems(page,layout)
	
	def buildUpperItems(self,container,containerLayout):
		layout = QHBoxLayout()
		layout.setSpacing(10)
		containerLayout.addLayout(layout)
		self.buildUpperLeftItems(container,layout)
		self.buildUpperRightItems(container,layout)
		
	def buildUpperLeftItems(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout,4)
		self.buildArtistItem(container,layout)
		self.buildAlbumItem(container,layout)
		self.buildGroupingItem(container,layout)
		self.buildComposerItem(container,layout)
		self.buildCommentsItem(container,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
	
	def buildUpperRightItems(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout,1)
		self.buildYearItem(container,layout)
		self.buildTrackItem(container,layout)
		self.buildDiscItem(container,layout)
		self.buildBPMItem(container,layout)
		self.buildArtworkItem(container,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
	
	def buildLowerItems(self,container,containerLayout):
		layout = QHBoxLayout()
		layout.setSpacing(10)
		containerLayout.addLayout(layout)
		self.buildLowerLeftItems(container,layout)
		self.buildLowerRightItems(container,layout)
	
	def buildLowerLeftItems(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		self.buildGenreItem(container,layout)
		self.buildVolumeItem(container,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
	
	def buildLowerRightItems(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		hlayout = QHBoxLayout()
		hlayout.setSpacing(10)
		layout.addLayout(hlayout)
		self.buildCompilationItem(container,hlayout)
		self.buildRatingItem(container,hlayout)
		self.buildEqualizerItem(container,layout)
		layout.addItem(QSpacerItem(1,1,QSizePolicy.Fixed,QSizePolicy.Expanding))
	
	def buildCheckItem(self,name,labelText,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel("<b>%s</b>" % unikode(labelText),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		checkName = name+"Check"
		editName = name+"Edit"
		checkItem = LGetMultiInfoCheckBox(container)
		self.__dict__[checkName] = checkItem
		hlayout.addWidget(checkItem)
		editItem = KLineEdit("",container)
		self.__dict__[editName] = editItem
		hlayout.addWidget(editItem)
		checkItem.setLineEditItem(editItem)

	def buildDualCheckItem(self,name,labelText,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(u"<b>%s</b>" % unikode(labelText),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		checkNumName = name+"NumCheck"
		editNumName = name+"NumEdit"
		checkCountName = name+"CountCheck"
		editCountName = name+"CountEdit"
		checkItem = LGetMultiInfoCheckBox(container)
		self.__dict__[checkNumName] = checkItem
		hlayout.addWidget(checkItem)
		editItem = KRestrictedLine(container,"","0123456789")
		self.__dict__[editNumName] = editItem
		editItem.setMinimumWidth(self.fontMetrics().maxWidth()*2)
		hlayout.addWidget(self.__dict__[editNumName])
		checkItem.setLineEditItem(editItem)
		label = QLabel(i18n("of"),container)
		hlayout.addWidget(label)
		checkItem = LGetMultiInfoCheckBox(container)
		self.__dict__[checkCountName] = checkItem
		hlayout.addWidget(checkItem)
		editItem = KRestrictedLine(container,"","0123456789")
		self.__dict__[editCountName] = editItem
		editItem.setMinimumWidth(self.fontMetrics().maxWidth()*2)
		hlayout.addWidget(editItem)
		checkItem.setLineEditItem(editItem)

	def buildArtistItem(self,container,containerLayout):
		self.buildCheckItem('artist',i18n('Artist'),container,containerLayout)
		
	def buildAlbumItem(self,container,containerLayout):
		self.buildCheckItem('album',i18n('Album'),container,containerLayout)
		
	def buildGroupingItem(self,container,containerLayout):
		self.buildCheckItem('grouping',i18n('Grouping'),container,containerLayout)
		
	def buildComposerItem(self,container,containerLayout):
		self.buildCheckItem('composer',i18n('Composer'),container,containerLayout)
		
	def buildCommentsItem(self,container,containerLayout):
		self.buildCheckItem('comment',i18n('Comments'),container,containerLayout)
		
	def buildYearItem(self,container,containerLayout):
		self.buildCheckItem('year',i18n('Year'),container,containerLayout)
		
	def buildTrackItem(self,container,containerLayout):
		self.buildDualCheckItem('track',i18n('Track'),container,containerLayout)
		
	def buildDiscItem(self,container,containerLayout):
		self.buildDualCheckItem('disc',i18n('Disc'),container,containerLayout)
		
	def buildBPMItem(self,container,containerLayout):
		self.buildCheckItem('bpm',i18n('BPM'),container,containerLayout)
		
	def buildArtworkItem(self,container,containerLayout):
		# XXX DSM build artwork well here
		pass
		
	def buildGenreItem(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(u"<b>%s</b)" % unikode(i18n("Genre")),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		self.genreCheck = LGetMultiInfoCheckBox(container)
		hlayout.addWidget(self.genreCheck)
		self.genreEdit = KComboBox(False,container)
		sortedGenres = musicGenres[:]
		sortedGenres.sort()
		for genre in sortedGenres:
			self.genreEdit.insertItem(genre)
		hlayout.addWidget(self.genreEdit,2)
		self.genreCheck.setComboBoxItem(self.genreEdit)
		
	def buildCompilationItem(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(u"<b>%s</b>" % unikode(i18n("Part of a Compilation")),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		self.compilationCheck = LGetMultiInfoCheckBox(container)
		hlayout.addWidget(self.compilationCheck)
		self.compilationEdit = KComboBox(False,container)
		self.compilationEdit.setFixedWidth(120)
		self.compilationEdit.insertItem(i18n('No'))
		self.compilationEdit.insertItem(i18n('Yes'))
		hlayout.addWidget(self.compilationEdit)
		self.compilationCheck.setComboBoxItem(self.compilationEdit)
		
	def buildRatingItem(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(u"<b>%s</b>" % unikode(i18n("Rating")),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		self.ratingCheck = LGetMultiInfoCheckBox(container)
		hlayout.addWidget(self.ratingCheck)
		self.ratingEdit = LRatingWidget(container)
		hlayout.addWidget(self.ratingEdit)
		self.ratingCheck.setRatingItem(self.ratingEdit)
		
	def buildVolumeItem(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(u"<b>%s</b>" % unikode(i18n("Volume Adjustment")),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		self.volumeCheck = LGetMultiInfoCheckBox(container)
		hlayout.addWidget(self.volumeCheck)
		self.volumeSlider = QSlider(Qt.Horizontal,container)
		self.volumeSlider.setMinValue(-100)
		self.volumeSlider.setMaxValue(100)
		self.volumeSlider.setTickmarks(QSlider.Below)
		self.volumeSlider.setTickInterval(10)
		hlayout.addWidget(self.volumeSlider)
		self.volumeCheck.setSliderItem(self.volumeSlider)
		
	def buildEqualizerItem(self,container,containerLayout):
		layout = QVBoxLayout()
		containerLayout.addLayout(layout)
		label = QLabel(u"<b>%s</b>" % unikode(i18n("Equalizer Preset")),container)
		layout.addWidget(label)
		hlayout = QHBoxLayout()
		layout.addLayout(hlayout)
		self.equalizerCheck = LGetMultiInfoCheckBox(container)
		hlayout.addWidget(self.equalizerCheck)
		self.equalizerEdit = KComboBox(False,container)
		for preset in equalizerPresets:
			self.equalizerEdit.insertItem(preset[0])
		hlayout.addWidget(self.equalizerEdit)
		hlayout.addItem(QSpacerItem(1,1,QSizePolicy.Expanding,QSizePolicy.Fixed))
		self.equalizerCheck.setComboBoxItem(self.equalizerEdit)
		
	def load(self):
		tags = None
		for track in self.tracks:
			tags = track.loadTagsInto(tags)
		#print "loading",tags
		if tags:
			items = ['artist','album','grouping','composer','comment',\
				'year','trackNum','trackCount','discNum','discCount','bpm']
			for item in items:
				itemEdit = self.__dict__[item+'Edit']
				itemEdit.blockSignals(True)
				item = tags.get(item,'')
				if type(item).__name__ in ['int','long']:
					item = str(item)
				itemEdit.setText(item)
				itemEdit.blockSignals(False)
			self.genreEdit.blockSignals(True)
			try: self.genreEdit.setCurrentText(str(tags.get('genre','')))
			except: pass
			self.genreEdit.blockSignals(False)
			self.volumeSlider.blockSignals(True)
			try: self.volumeSlider.setValue(tags.get('volume',0))
			except: pass
			self.volumeSlider.blockSignals(False)
			self.equalizerEdit.blockSignals(True)
			try: self.equalizerEdit.setCurrentText(str(tags.get('equalizer','')))
			except: pass
			self.equalizerEdit.blockSignals(False)
			self.compilationEdit.blockSignals(True)
			try:
				self.compilationEdit.setCurrentText('')
				if tags.get('compilation',''):
					self.compilationEdit.setCurrentText('Yes')
				else:
					self.compilationEdit.setCurrentText('No')
			except: pass
			self.compilationEdit.blockSignals(False)
			self.ratingCheck.blockSignals(True)
			try:
				self.ratingEdit.setRating(tags['rating'])
				self.ratingCheck.setChecked(False) # XXX DSM PyQt does not block python signals in this version
			except: pass
			self.ratingCheck.blockSignals(False)
	
	def unload(self):
		tags = {}
		items = ['artist','album','grouping','composer','comment',\
			'year','trackNum','trackCount','discNum','discCount','bpm']
		for item in items:
			if self.__dict__[item+'Check'].isChecked():
				tags[item] = str(self.__dict__[item+'Edit'].text())
		if self.genreCheck.isChecked():
			tags['genre'] = str(self.genreEdit.currentText())
		if self.volumeCheck.isChecked():
			tags['volume'] = self.volumeSlider.value()
		if self.equalizerCheck.isChecked():
			tags['equalizer'] = str(self.equalizerEdit.currentText())
		if self.compilationCheck.isChecked():
			tags['compilation'] = self.compilationEdit.currentText()==i18n('Yes')
		if self.ratingCheck.isChecked():
			tags['rating'] = self.ratingEdit.rating()
		# XXX DSM handle artwork
		#print tags
		return tags

if __name__=="__main__":
	app = KApplication(sys.argv,"test")
	QObject.connect(app,SIGNAL('lastWindowClosed()'),app,SLOT('quit()'))
	dialog = LGetMultiInfoDialog()
	app.setMainWidget(dialog)
	dialog.show()
	app.exec_loop()
