#
# "@(#) $Id: LLibraryViewItem.py,v 1.18 2004/08/04 19:36:54 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from kdeemul import *
from LMusicPlayer import *
from human import *
from LCheckListViewItem import *
from LTrackViewItemObject import *
from LsongsPaths import *
from Library import *
from utils import *

def _get(a,b):
	if a==None: return b
	return a

_star = None

def star():
	global _star
	if _star==None:
		_star = QPixmap(art("labelstar.png"))
	return _star


_playingIcon = None
def playingIcon():
	global _playingIcon
	if _playingIcon==None:
		_playingIcon = QPixmap(art("labelcurrentplaying.png"))
	return _playingIcon

_pausedIcon = None

def pausedIcon():
	global _pausedIcon
	if _pausedIcon==None:
		_pausedIcon = QPixmap(art("labelcurrentpause.png"))
	return _pausedIcon

class LLibraryViewItem(LCheckListViewItem):
	def __init__(self,container,track):
		LCheckListViewItem.__init__(self,container)
		self._track = track
		self.trackItem = LTrackViewItemObject(self)
		self.status = 'idle'
		self.setDragEnabled(True)
		self.setColumnIndices()
		self.loadTrackInfo()
		self.dummyKey = QString(" ")

	def die(self):
		#print "item dying"
		if self.trackItem:
			self.trackItem.trackItem = None
			self.trackItem = None

	def setColumnIndices(self):
		self.titleColumn = 1
		self.durationColumn = 2
		self.artistColumn = 3
		self.albumColumn = 4
		self.genreColumn = 5
		self.ratingColumn = 6
		self.playCountColumn = 7
		self.playDateColumn = 8
	
	def track(self):
		return self._track

	def loadTrackInfo(self):
		self.titleKey = _get(self._track.title,'<No name>')
		self.setText(self.titleColumn,self.titleKey)
		self.titleKey = QString(self.titleKey.upper())
		self.durationKey = self._track.totalTime
		self.setText(self.durationColumn,humanDuration(self.durationKey))
		if self.durationKey:
			self.durationKey = QString("%010d" % self.durationKey)
		else: self.durationKey = QString('ZZZ')
		self.artistKey = _get(self._track.artist,'')
		self.setText(self.artistColumn,self.artistKey)
		if self.artistKey[0:4]=="The ":
			self.artistKey = self.artistKey[4:]
		if len(self.artistKey)==0: self.artistKey = 'Z'
		else: self.artistKey = 'A%s' % self.artistKey.upper()
		self.albumKey = _get(self._track.album,'')
		self.setText(self.albumColumn,self.albumKey)
		if len(self.albumKey)==0: self.albumKey = 'Z'
		else: self.albumKey = 'A%s' % self.albumKey.upper()
		trackNum = self._track.trackNum
		if trackNum!=None:
			self.albumKey = unikode(self.albumKey.upper())+unikode((' T%05d' % trackNum))
			self.artistKey = unikode(self.artistKey.upper())+unikode(self.albumKey)
		self.artistKey = QString(self.artistKey)
		self.albumKey = QString(self.albumKey)
		self.genreKey = _get(self._track.genre,'')
		self.setText(self.genreColumn,self.genreKey)
		self.genreKey = QString(self.genreKey)
		if self.ratingColumn:
			self.rating = self._track.rating # drawn specially
			if self.rating: self.ratingKey = QString(self.rating)
			else: self.ratingKey = QString(0)
			self.setText(self.ratingColumn,'')
		if self.playCountColumn:
			self.playCountKey = self._track.playCount
			if self.playCountKey:
				self.setText(self.playCountColumn,str(self.playCountKey))
			else:
				self.setText(self.playCountColumn,"")
				self.playCountKey = 0
			self.playCountKey = QString("%06d" % self.playCountKey)
		if self.playDateColumn:
			self.playDateKey = humanTimeStamp(self._track.playDate)
			self.setText(self.playDateColumn,self.playDateKey)
			self.playDateKey = QString(self.playDateKey)
		self.setRenameEnabled(self.titleColumn,1)
		self.setRenameEnabled(self.artistColumn,1)
		self.setRenameEnabled(self.albumColumn,1)
		#self.setRenameEnabled(self.genreColumn,1)
		self.setOn(self._track.enabled)
		self.setPlayingIcon()
	
	def setPlayingIcon(self):
		status = self._track.playStatus
		if self.status!=status:
			if status=='playing':
				self.setPixmap(0,playingIcon())
			elif status=='paused':
				self.setPixmap(0,pausedIcon())
			else:
				self.setPixmap(0,QPixmap())
			self.status = status

	def key(self,column,ascending):
		#print "hit LLibraryView key",column
		if column==self.titleColumn:
			return self.titleKey
		elif column==self.durationColumn:
			return self.durationKey
		elif column==self.artistColumn:
			return self.artistKey
		elif column==self.albumColumn:
			return self.albumKey
		elif column==self.genreColumn:
			return self.genreKey
		elif column==self.ratingColumn:
			return self.ratingKey
		elif column==self.playCountColumn:
			return self.playCountKey
		elif column==self.playDateColumn:
			return self.playDateKey
		else:
			return self.dummyKey

	def okRename(self,column):
		oldValue = unikode(self.text(column))
		LCheckListViewItem.okRename(self,column)
		newValue = unikode(self.text(column))
		if oldValue!=newValue:
			newValue = newValue.strip()
			if column==self.titleColumn:
				if len(newValue)==0:
					KMessageBox.error(None,i18n("Track title must not be empty"),i18n("Bad Track Title"))
					self.setText(self.titleColumn,oldValue)
					self.startRename(self.titleColumn)
				else:
					self._track.loadTagsFrom({'title':newValue})
			elif column==self.artistColumn:
				self._track.loadTagsFrom({'artist':newValue})
			elif column==self.albumColumn:
				self._track.loadTagsFrom({'album':newValue})
			elif column==self.genreColumn:
				self._track.loadTagsFrom({'genre':newValue})

	def trackID(self):
		return self._track.trackID

	def contentsMouseDoubleClickEvent(self,event):
		self.play()

	def play(self):
		playlist = self.listView().playlist
		self._track.play(playlist)
	
	def stateChange(self,value):
		self._track.enabled = bool(value)

	def changedAttribute(self,attribute,oldValue,newValue):
		#print "changed",attribute,"from",oldValue,"to",newValue
		if attribute in ['title','artist','album','contentType','rating','playCount','playDate','enabled']:
			self.loadTrackInfo()
			if attribute=='rating':
				self.repaint()
		elif attribute=='playStatus':
			self.setPlayingIcon()
	
	def paintCell(self,p,cg,column,width,alignment):
		LCheckListViewItem.paintCell(self,p,cg,column,width,alignment)
		if column==self.ratingColumn and self.rating: # rating column
			for i in xrange(0,self.rating):
				p.drawPixmap(QPoint(i*13+3,2),star())

	def clickedRatingAt(self,px):
		if self.isSelected():
			stars = ((px-3)/13)+1
			if stars<0:
				stars = 0
			elif stars>5:
				stars = 5
			self._track.loadTagsFrom({'rating':stars})

	def getInfo(self):
		trackIDs = self.listView().selectedTrackIDs()
		if trackIDs and len(trackIDs)>0:
			if len(trackIDs)==1:
				self._track.getInfo()
			else:
				self.listView().getMultiInfoForTrackIDs(trackIDs)
	
	def pause(self):
		self._track.pause()
	
	def showFile(self):
		self._track.showLocation()
	
	def resetPlayCount(self):
		tracks = self.listView().selectedTracks()
		for track in tracks:
			track.playCount = 0
	
	def enable(self):
		self.doEnable(True)
		
	def disable(self):
		self.doEnable(False)
	
	def doEnable(self,enable):
		tracks = self.listView().selectedTracks()
		for track in tracks:
			track.enabled = enable

	def moveToTrash(self):
		library = Library.mainLibrary()
		trash = library.playlistWithName("Trash")
		trackIDs = self.listView().selectedTrackIDs()
		#print "moving",trackIDs,"to",trash
		trash.addTrackIDs(trackIDs)
		#print "removing tracks from all playlists"
		library.removeTrackIDsFromAllPlaylists(trackIDs)

	def contextMenu(self,pos,col):
		menu = QPopupMenu()
		self.addContextMenuItems(menu)
		item = menu.exec_loop(pos)
	
	def addContextMenuItems(self,menu):
		self.addStandardContextMenuItems(menu)
		self.addCustomContextMenuItems(menu)
	
	def addStandardContextMenuItems(self,menu):
		status = self._track.playStatus
		if status=='playing':
			menu.insertItem(i18n("Pause"),self.pause)
		else:
			menu.insertItem(i18n("Play"),self.play)
		menu.insertItem(i18n("Get Info..."),self.getInfo)
		menu.insertItem(i18n("Show Song File"),self.showFile)
		menu.insertItem(i18n("Reset Play Count"),self.resetPlayCount)
		menu.insertSeparator()
		menu.insertItem(i18n("Create Playlist for this Artist"),self.createArtistPlaylist)
		menu.insertItem(i18n("Create Playlist for this Album"),self.createAlbumPlaylist)
		menu.insertSeparator()
		menu.insertItem(i18n("Enable"),self.enable)
		menu.insertItem(i18n("Disable"),self.disable)
	
	def addCustomContextMenuItems(self,menu):
		menu.insertItem(i18n("Move to Trash"),self.moveToTrash)
	
	def createArtistPlaylist(self):
		if self._track.artist:
			self.listView().createArtistPlaylist(unikode(self._track.artist))
	
	def createAlbumPlaylist(self):
		if self._track.album:
			self.listView().createAlbumPlaylist(unikode(self._track.album))

