#
# "@(#) $Id: LMusicCDMounter.py,v 1.7 2004/07/21 17:54:18 duane Exp $"
#
# This work is released under the GNU GPL, version 2 or later.
#
from qt import *
import os, time, re
from MP3Track import *
from CDTrack import *
import cdinfo

_curThread = None
_altThread = None
_mutex = QMutex() # used to pass info between threads
_command = ""
_response = ""

class LMusicCDMounterEvent(QCustomEvent):
	def __init__(self,status = None):
		QCustomEvent.__init__(self,QEvent.User+5)
		self._status = status
	
	def status(self):
		return self._status

class LMusicCDMounterThread(QThread):
	def __init__(self,library):
		QThread.__init__(self)
		global _altThread
		_altThread = self
		self.library = library
		library.clearTracks()
	
	def run(self):
		from CDROM import CDROM
		cancelled = False
		self.emitEvent({'Source':'CDMounter','Status':'Begin'})
		index = 0
		self.library.loadInfo()
		total = CDROM.singleton().numTracks()
		timeOffset = 0
		while index<total and not cancelled:
			cmd = self.getCommand()
			if cmd=='stop':
				cancelled = True
			else:
				track = self.importTrack(index,total)
				track.startTime = timeOffset
				timeOffset = timeOffset+track.totalTime
				self.msleep(10)
			index = index+1
		self.emitEvent({'Source':'CDMounter','Status':'End','Interrupted':cancelled})
		if cancelled:
			self.putResponse('stopped')
		self.die()
	
	def importTrack(self,index,total):
		self.emitEvent({'Source':'CDMounter','Status':'Importing','currentTime':index,'totalTime':total})
		track = CDTrack(self.library)
		track.loadFromCD(index+1)
		track.trackCount = total
		track.trackID = self.library.uniqueTrackID()
		self.library.addTrack(track)
		if track.album:
			self.library.name = track.album
		else:
			self.library.name = "CD"
		self.emitEvent({'Source':'CDMounter','Status':'Imported','currentTime':index,'totalTime':total,'Track':track})
		return track

	def die(self):
		global _mutex,_curThread
		_mutex.lock()
		_curThread = None
		_mutex.unlock()
	
	def importFile(self,uri,index,total):
		(srcPapth,fileName) = os.path.split(uri)
		self.emitEvent({'Source':'CDMounter','Status':'Importing','Filename':fileName,'currentTime':index,'totalTime':total})
		track = MP3Track()
		track.file = uri
		self.emitEvent({'Source':'CDMounter','Status':'Imported','Track':track,'currentTime':index,'totalTime':total,'Library':self.library,'Playlist':self.playlist})

	def emitEvent(self,status):
		event = LMusicCDMounterEvent(status)
		QApplication.postEvent(LMusicCDMounter.singleton(),event)

	def getCommand(self):
		global _command
		global _mutex
		_mutex.lock()
		cmd = _command
		_command = ''
		_mutex.unlock()
		return cmd

	def putResponse(self,resp):
		global _response
		global _mutex
		_mutex.lock()
		_response = resp
		_mutex.unlock()

class LMusicCDMounter(QObject):
	def __init__(self):
		QObject.__init__(self)
	
	def customEvent(self,e):
		global _status
		_status = e.status()
		#print "custom event",_status
		state = _status['Status']
		self.emit(PYSIGNAL('status'),(_status,None))
		if state=='End':
			self.emit(PYSIGNAL('cdInserted'),(None,None))
	
	def static_mount(library):
		global _curThread
		disc_id = cdinfo.disc_id()
		if disc_id!=None:
			_curThread = LMusicCDMounterThread(library)
			_curThread.start()
		
	mount = staticmethod(static_mount)
	
	def static_singleton():
		global _cdMounterSingleton
		return _cdMounterSingleton

	singleton = staticmethod(static_singleton)

	def static_killCurrentCDMounter():
		global _curThread,_altThread
		if _curThread:
			print "killing CD Mounter"
			LMusicCDMounter.putCommand("stop")
			t = long(time.time())+5
			while t>=long(time.time()):
				if LMusicCDMounter.getResponse()=='stopped':
					break
				time.sleep(0.1)
			if _curThread!=None and not _curThread.wait(2000):
				print "LMusicCDMounter: thread is hung, terminating"
				_curThread.terminate()
				print "LMusicCDMounter: waiting for termination"
				try:
					if _curThread and _curThread.wait(2000):
						print "LMusicCDMounter: thread hopelessly hung"
				except: pass
				_curThread = None
				_altThread = None
			else:
				print "LMusicCDMounter: thread died cleanly"

	killCurrentCDMounter = staticmethod(static_killCurrentCDMounter)

	def static_getResponse():
		global _response
		global _mutex
		_mutex.lock()
		cmd = _response
		_mutex.unlock()
		return cmd
	getResponse = staticmethod(static_getResponse)

	def static_putCommand(cmd):
		global _command
		global _response
		global _mutex
		_mutex.lock()
		_command = cmd
		_response = ''
		_mutex.unlock()
	putCommand = staticmethod(static_putCommand)

_cdMounterSingleton = LMusicCDMounter()
